@@ -4,16 +4,16 @@ package Text::ASCIITable;
 @ISA=qw(Exporter);
 @EXPORT = qw();
 @EXPORT_OK = qw();
-$VERSION = '0.05';
+$VERSION = '0.06';
 use Exporter;
 use strict;
 use Carp;
+use Text::Aligner;
 
-sub new {
-  my $self = {tbl_cols => [],tbl_rows => [],tbl_alignright => []};
-  bless $self;
-  return $self;
-}
+# Determine if Text::Wrap is installed
+my $hasWrap;
+if (eval { require Text::Wrap }) { use Text::Wrap; $hasWrap=1; }
+else { $hasWrap=0; }
 
 =head1 NAME
 
@@ -36,6 +36,25 @@ text to your console or other fixed-size displays.
 
 =head1 FUNCTIONS
 
+=head2 new(options)
+
+Initialize a new table. You can specify output-options. For more options, check out the usage for setOptions(name,value)
+
+  Usage:
+  $t = new Text::ASCIITable;
+  
+  Or with options:
+  $t = new Text::ASCIITable({ hide_Lastline => 1, reportErrors => 0});
+
+=cut
+
+sub new {
+  my $self = {tbl_cols => [],tbl_rows => [],tbl_align => {},options => $_[1] || { }};
+  $self->{options}{reportErrors} = 1; # default setting
+  bless $self;
+  return $self;
+}
+
 =head2 setCols(@cols)
 
 Define the columns for the table(compare with <TH> in HTML). For example C<setCols(['Id','Nick','Name'])>.
@@ -46,10 +65,10 @@ B<Note> that you cannot add Cols after you have added a row.
 sub setCols {
   my $self = shift;
   my $tmp = shift;
-  do { print STDERR Carp::shortmess "setCols needs an array"; return 1; } unless defined($tmp);
+  do { $self->reperror("setCols needs an array"); return 1; } unless defined($tmp);
   my @cols = @{$tmp};
-  do { print STDERR Carp::shortmess "setCols needs an array"; return 1; } unless scalar(@cols) != 0;
-  do { print STDERR Carp::shortmess "Cannot edit cols at this state"; return 1; } unless scalar(@{$self->{tbl_rows}}) == 0;
+  do { $self->reperror("setCols needs an array"); return 1; } unless scalar(@cols) != 0;
+  do { $self->reperror("Cannot edit cols at this state"); return 1; } unless scalar(@{$self->{tbl_rows}}) == 0;
   @{$self->{tbl_cols}} = @cols;
   return undef;
 }
@@ -62,8 +81,8 @@ Add a column to the columnlist. This still can't be done after you have added a
 
 sub addCol {
   my ($self,$col) = @_;
-  do { print STDERR Carp::shortmess "addCol needs a string"; return 1; } unless defined($col);
-  do { print STDERR Carp::shortmess "Cannot add cols at this state"; return 1; } if (scalar(@{$self->{tbl_rows}}) != 0);
+  do { $self->reperror("addCol needs a string"); return 1; } unless defined($col);
+  do { $self->reperror("Cannot add cols at this state"); return 1; } if (scalar(@{$self->{tbl_rows}}) != 0);
   push @{$self->{tbl_cols}},$col;
   return undef;
 }
@@ -71,43 +90,99 @@ sub addCol {
 =head2 addRow(@collist)
 
 Adds one row to the table. This must be an array of strings. If you defined 3 columns. This array must
-have 3 items in it. And so on. Should be self explanatory.
+have 3 items in it. And so on. Should be self explanatory. The strings can contain newlines.
 
 =cut
 
 sub addRow {
   my $self = shift;
-  do { print STDERR Carp::shortmess "Received too few columns"; return 1; } if scalar(@_) < scalar(@{$self->{tbl_cols}});
-  do { print STDERR Carp::shortmess "Received too many columns"; return 1; } if scalar(@_) > scalar(@{$self->{tbl_cols}});
-  push @{$self->{tbl_rows}}, [@_];
+  do { $self->reperror("Received too few columns"); return 1; } if scalar(@_) < scalar(@{$self->{tbl_cols}});
+  do { $self->reperror("Received too many columns"); return 1; } if scalar(@_) > scalar(@{$self->{tbl_cols}});
+  my (@in,@out,@lines,$max);
+
+  # Word wrapping:
+  if ($hasWrap) {
+    foreach my $c (0..(scalar(@_)-1)) {
+      my $width = $self->{tbl_width}{@{$self->{tbl_cols}}[$c]};
+      if ($width) {
+        $Text::Wrap::columns = $width;
+        @in[$c] = wrap('', '', @_[$c]);
+      } else {
+        @in[$c] = @_[$c];
+      }
+    }
+  } else { @in = @_; }
+
+  # Multiline support:
+  @lines = map { [ split(/\n/,$_) ] } @in;
+  foreach (@lines) {
+    $max = scalar(@{$_}) if scalar(@{$_}) > $max;
+  }
+  foreach my $num (0..($max-1)) {
+    my @tmp;
+    foreach (@lines) {
+      push @tmp,iif(@{$_}[$num],'');
+    }
+    push @out, [ @tmp ];
+  }
+
+  # Add row(s)
+  push @{$self->{tbl_rows}}, @out;
+
   return undef;
 }
 
-=head2 alignColRight($col)
+=head2 alignCol($col,$direction)
 
-Given a columnname, it aligns all data to the right in the table. This looks nice on numerical displays
-in a column. The column names in the table will not be unaffected by the alignment.
+Given a columnname, it aligns all data to the given direction in the table. This looks nice on numerical displays
+in a column. The column names in the table will not be unaffected by the alignment. Possible directions is: left,
+center and right. (Hint: It is often very useful to align numbers to the right, and text to the left.)
 
 =cut
 
+# backwardscompatibility, deprecated
 sub alignColRight {
   my ($self,$col) = @_;
-  do { print STDERR Carp::shortmess "alignColRight needs a string"; return 1; } unless defined($col);
-  do { print STDERR Carp::shortmess "Could not find '$col' in columnlist"; return 1; } unless defined(finn($col,$self->{tbl_cols}));
-  return undef if defined(finn($col,$self->{tbl_alignright}));
-  push @{$self->{tbl_alignright}}, $col;
+  do { $self->reperror("alignColRight is missing parameter(s)"); return 1; } unless defined($col);
+  return $self->alignCol($col,'right');
+}
+
+sub alignCol {
+  my ($self,$col,$direction) = @_;
+  do { $self->reperror("alignCol is missing parameter(s)"); return 1; } unless defined($col) && defined($direction);
+  do { $self->reperror("Could not find '$col' in columnlist"); return 1; } unless defined(&find($col,$self->{tbl_cols}));
+  $self->{tbl_align}{$col} = $direction;
 
   return undef;
 }
 
-# now the real stuff
+=head2 setColWidth($col,$width)
+
+Wordwrapping. Set a max-width(in chars) for a column.
+
+ Usage:
+  $t->setColWidth('Description',30);
+
+=cut
+
+sub setColWidth {
+  my ($self,$col,$width) = @_;
+  do { $self->reperror("setColWidth is missing parameter(s)"); return 1; } unless defined($col) && defined($width);
+  do { $self->reperror("Could not find '$col' in columnlist"); return 1; } unless defined(&find($col,$self->{tbl_cols}));
+  do { $self->reperror("Text::Wrap not installed. Please install from CPAN"); return 1; } unless $hasWrap;
+  $self->{tbl_width}{$col} = int($width);
+
+  return undef;
+}
+
+# drawing etc, below
 
 sub getColWidth {
   my ($self,$colname) = @_;
-  my $pos = finn($colname,$self->{tbl_cols});
+  my $pos = &find($colname,$self->{tbl_cols});
   my $maxsize = length($colname);
 
-  do { print STDERR Carp::shortmess "Could not find '$colname' in columnlist"; return 1; } unless defined($pos);
+  do { $self->reperror("Could not find '$colname' in columnlist"); return 1; } unless defined($pos);
   for my $row (@{$self->{tbl_rows}}) {
     $maxsize = length(@{$row}[$pos]) if (length(@{$row}[$pos]) > $maxsize);
   }
@@ -133,7 +208,7 @@ sub getTableWidth {
 
 sub drawLine {
   my ($self,$start,$stop,$line,$delim) = @_;
-  do { print STDERR Carp::shortmess "Missing reqired parameters"; return 1; } unless defined($stop);
+  do { $self->reperror("Missing reqired parameters"); return 1; } unless defined($stop);
   $line = defined($line) ? $line : '-'; 
   $delim = defined($delim) ? $delim : '+'; 
 
@@ -150,29 +225,46 @@ sub drawLine {
 
     $contents .= $delim if ($i != scalar(@{$self->{tbl_cols}}) - 1);
   }
-  return $contents .= $stop."\n";
+  return $contents.$stop."\n";
+}
+
+=head2 setOptions(name,value)
+
+Use this to set options like: hide_FirstLine,hide_HeadLine,hide_HeadRow,hide_LastLine.
+
+  $t->setOptions('hide_HeadLine',1);
+
+=cut
+
+sub setOptions {
+  my ($self,$name,$value) = @_;
+  my $old = $self->{options}{$name} || undef;
+  $self->{options}{$name} = $value;
+  return $old;
 }
 
 sub drawRow {
   my ($self,$row,$allowalign,$start,$stop,$delim) = @_;
-  do { print STDERR Carp::shortmess "Missing reqired parameters"; return 1; } unless defined($row);
-  $allowalign = defined($allowalign) ? $allowalign : 1;
-  $delim = defined($delim) ? $delim : '|';
+  do { $self->reperror("Missing reqired parameters"); return 1; } unless defined($row);
+  $allowalign = &iif($allowalign,1);
+  $delim = &iif($delim,'|');
 
   my $contents = $start;
   for (my $i=0;$i<scalar(@{$row});$i++) {
     my $text = @{$row}[$i];
 
-    if ($allowalign == 1 && scalar(@{$self->{tbl_alignright}}) && defined(finn(@{$self->{tbl_cols}}[$i],$self->{tbl_alignright}))) {
-      $contents .= ' ' x ($self->getColWidth(@{$self->{tbl_cols}}[$i]) - length($text) - 1);
-      $contents .= $text.' ';
+    if ($allowalign == 1 && defined($self->{tbl_align}{@{$self->{tbl_cols}}[$i]})) {
+      my $align = Text::Aligner->new(&iif($self->{tbl_align}{@{$self->{tbl_cols}}[$i]},'left'));
+      $align->alloc('-' x ($self->getColWidth(@{$self->{tbl_cols}}[$i]) - 2));
+      $contents .= ' '.$align->justify($text).' ';
     } else {
-      $contents .= ' '.$text;
-      $contents .= ' ' x ($self->getColWidth(@{$self->{tbl_cols}}[$i]) - length($text) - 1);
+      my $align = Text::Aligner->new('left');
+      $align->alloc('-' x ($self->getColWidth(@{$self->{tbl_cols}}[$i]) - 2));
+      $contents .= ' '.$align->justify($text).' ';
     }
     $contents .= $delim if ($i != scalar(@{$row}) - 1);
   }
-  $contents .= $stop."\n";
+  return $contents.$stop."\n";
 }
 
 =head2 draw([@topdesign,@toprow,@middle,@middlerow,@bottom])
@@ -282,29 +374,30 @@ sub draw {
   my ($mrstart,$mrstop,$mrdelim) = defined($middlerow) ? @{$middlerow} : undef;
   my ($bstart,$bstop,$bline,$bdelim) = defined($bottom) ? @{$bottom} : undef;
   my $contents="";
-  $contents .= $self->drawLine(iif($tstart,'.'),iif($tstop,'.'),$tline,$tdelim);
-  $contents .= $self->drawRow($self->{tbl_cols},0,iif($trstart,'|'),iif($trstop,'|'),iif($trdelim,'|'));
-  $contents .= $self->drawLine(iif($mstart,' >'),iif($mstop,'< '),$mline,$mdelim);
+  $contents .= $self->drawLine(&iif($tstart,'.'),&iif($tstop,'.'),$tline,$tdelim) unless $self->{options}{hide_FirstLine};
+  $contents .= $self->drawRow($self->{tbl_cols},0,&iif($trstart,'|'),&iif($trstop,'|'),&iif($trdelim,'|')) unless $self->{options}{hide_HeadRow};
+  $contents .= $self->drawLine(&iif($mstart,' >'),&iif($mstop,'< '),$mline,$mdelim) unless $self->{options}{hide_HeadLine};
   for (@{$self->{tbl_rows}}) {
-    $contents .= $self->drawRow($_,1,iif($mrstart,'|'),iif($mrstop,'|'),iif($mrdelim,'|'));
+    $contents .= $self->drawRow($_,1,&iif($mrstart,'|'),&iif($mrstop,'|'),&iif($mrdelim,'|'));
   }
-  $contents .= $self->drawLine(iif($bstart,"'"),iif($bstop,"'"),$bline,$bdelim);
+  $contents .= $self->drawLine(&iif($bstart,"'"),&iif($bstop,"'"),$bline,$bdelim) unless $self->{options}{hide_LastLine};
   return $contents;
 }
-sub iif {
-  my ($if,$els) = @_;
-  return $els unless defined $if;
-  return $if
+
+# nifty subs
+
+sub reperror {
+  my $self = shift;
+  print STDERR Carp::shortmess(shift) if $self->{options}{reportErrors};
 }
-# couldn't find a better way to search in an array, than to make this function. Please tell me the right way..
-sub finn {
-  my $naal = shift;
-  my $arr = shift;
-  return undef unless defined $arr;
-  for (my $i=0;$i < scalar(@{$arr});$i++) {
-    if (@{$arr}[$i] eq $naal) {
-      return $i;
-    }
+sub iif {
+  return defined($_[0]) ? $_[0] : $_[1];
+}
+# Couldn't find a better way to search in an array, than to make this function. Please tell me the right way..
+sub find {
+  return undef unless defined $_[1];
+  for (0..scalar(@{$_[1]})) {
+    return $_ if @{$_[1]}[$_] eq $_[0];
   }
   return undef;
 }
@@ -316,7 +409,7 @@ __END__
 
 =head1 REQUIRES
 
-Exporter, Carp
+Exporter, Carp, Text::Aligner, Text::Wrap
 
 =head1 AUTHOR
 
